/* 
 * compat.h -- compatibility headers for rlc.
 *
 * This file is part of
 *
 * rlc -- Reliable Multicast data Distribution Protocol
 * 
 * (C) 1996, 1997, 1998 Luigi Rizzo and Lorenzo Vicisano
 *     (luigi@iet.unipi.it, vicisano@cs.ucl.ac.uk)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Luigi Rizzo,
 *      Lorenzo Vicisano and other contributors.
 * 4. Neither the name of the Authors nor the names of other contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#ifndef RLC__COMPAT_H
#define RLC__COMPAT_H

#include<sys/time.h>

#ifdef IRIX
#include<sys/endian.h>
#endif

#ifdef HPUX
#include<sys/param.h>
#endif

#ifdef FreeBSD
#include<machine/endian.h>
#endif

#ifdef Linux
#include<endian.h>
#endif

#ifdef SunOS
#include<sys/types.h>
#endif

#define min(x,y)	(((x)>(y))?(y):(x))
#define max(x,y)	(((x)<(y))?(y):(x))

#ifdef FreeBSD
#include <string.h>
#else
#define bcopy(s, d, siz)	memcpy((d), (s), (siz))
#define bzero(d, siz)	memset((d), '\0', (siz))
#endif

#if (defined SunOS)
int gethostname (char *, int);
#endif

#if (!(defined Linux))
#if ((defined SunOS) && (OSMVER < 5))
void perror(char *);
#endif
#endif

/* endianess... */
#ifdef BYTE_ORDER
#if (BYTE_ORDER == LITTLE_ENDIAN)
#define _LITTLE_ENDIAN
#elif BYTE_ORDER == BIG_ENDIAN
#define  _BIG_ENDIAN
#endif
#else /* BYTE_ORDER */
#if ( !defined(_LITTLE_ENDIAN) && !defined(_BIG_ENDIAN) )
#ifdef SunOS
#define  _BIG_ENDIAN
/* #error We have SunOS assuming BIG_ENDIAN */
#else
#error	endianness not defined! check compat.h
#endif
#endif
#endif /* BYTE_ORDER */

/* integer types */
typedef char		i8;		/*  8 bit signed int */
typedef short		i16;		/* 16 bit signed int */
typedef int		i32;		/* 32 bit signed int */
#if defined(__alpha)
typedef long		i64;		/* 64 bit signed int */
#endif

typedef unsigned char	ui8;		/*  8 bit unsigned int */
typedef unsigned short	ui16;		/* 16 bit unsigned int */
typedef unsigned short	n16;		/* 16 bit net format */
typedef unsigned int	ui32;		/* 32 bit unsigned int */
typedef unsigned int	n32;		/* 32 bit net format */
#if defined(__alpha)
typedef unsigned long	ui64;		/* 64 bit unsigned int */
#endif

typedef ui32		timest_t;	/* used for timestamp in usec */

#endif  /* RLC__COMPAT_H */
