/*
 * header file for rlc
 * 
 * This file is part of
 *
 * rlc -- Reliable Multicast data Distribution Protocol
 * 
 * (C) 1996, 1997, 1998 Luigi Rizzo and Lorenzo Vicisano
 *     (luigi@iet.unipi.it, vicisano@cs.ucl.ac.uk)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Luigi Rizzo,
 *      Lorenzo Vicisano and other contributors.
 * 4. Neither the name of the Authors nor the names of other contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef RLC_H
#define RLC_H

#include "compat.h"	/* this is for the endianness */

/* default values */
#define TTL		255	/*+ default ttl of transmitted packets +*/
#define NLAY		5	/*+ default number of layers +*/
#define	DATA_SIZE	256	/*+ default payload size +*/
#define THE_W		8
#define THE_Y		2
#define THE_P		1
#define DEFAULT_DPORT	"224.5.5.6/5657"
#define L0_PERIOD	125000	/*+ default base period +*/

/* limit values */
#define MAX_LAYERS     		10
#define MAX_RX_SOCKS   		512
#define MIN_PAYLOAD_SIZE	128
#define MAX_PAYLOAD_SIZE	(1500-DATA_H_SIZE)
#if 1		/* XXX check this */
#define MAX_L0_PERIOD		12500000
#else
#define MAX_L0_PERIOD		125000
#endif
#define MAX_LAYERS     10
/* that should disappare */
#define ADDRLEN 32

/*++++
 * data packet
 *
 
  0                   1                   2                   3
  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 |  V=0  |  T  |x|    flags      |            MAGIC #            |
 +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 |                           source id                           |
 +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 |             LSN               |              PN               |
 +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 |  num. layers  |  band. prof.  |   band. profile parameters    |
 +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 |                              data                             |
 |                              ....                             |
 +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

 *
 *
 ++++*/

#define RLC_VERSION	1
#define RLC_MAGIC	0x5afe

/* packet types */
#define RLC_DATA_T	0	/* data packet (down-link) */
#define RLC_WRMP_T	1	/* warmup paket */
#define RLC_INFO_T	2	/* info paket */
#define RLC_APPL_T	3	/* application specific info */


typedef struct _rlc_h {
#ifdef _LITTLE_ENDIAN
    ui8   pad:1,
	  type:3,	/* packet type */
    	  vers:4;	/* version */
#endif
#ifdef _BIG_ENDIAN
    ui8   vers:4,	/* version */
    	  type:3,	/* packet type */
	  pad:1;
#endif
#define RLC_F_IN_BURST		0x1
#define RLC_F_S_F_BURST		0x2
#define RLC_F_L_BURST		0x4
#define RLC_F_P_BURST		0x8
#define RLC_F_HART_BEAT		0x10
#define RLC_F_CONS_ADDR		0x20
#define RLC_F_EDGE_TRIG		0x40
    ui8   flags;	/* various flags */
    ui16  magic;	/* magic number */
    ui32  suid;		/* source ID */
    ui16  lsn;		/* sequence number within the layer */
    ui16  pn;		/* inter layer pattern sequence number */
#define	BT_EXP_1	0
    ui8   layer_n;	/* number of layers */
    ui8	  bp;
    ui16  bpp;
} rlc_h;

#define FILL_D_H(p,t_lsn,t_pn,t_suid,f,		\
	l_n, b_p,valid_data) {			\
	(p).vers=RLC_VERSION;			\
	(p).magic=htons(RLC_MAGIC);		\
	if (valid_data) (p).type=RLC_DATA_T;	\
	else (p).type=RLC_WRMP_T;		\
	(p).lsn=(t_lsn);			\
	(p).pn=(t_pn);				\
	(p).suid=(t_suid);			\
	(p).flags=(f);				\
	(p).layer_n=(l_n);			\
	(p).bp=(b_p);				\
	(p).bpp=0;				\
    }
    
#define DATA_H_SIZE sizeof(rlc_h)

#define MAX_PKT_LEN	(DATA_H_SIZE+MAX_PAYLOAD_SIZE)

#endif  /* RLC_H */
